require 'fluent/config/types'
require 'fluent/plugin/out_file'
require 'fluent/env'

module Fluent
  class FileLimitOutput < FileOutput
    Plugin.register_output('file-limit', self)

    config_param :file_number_limit, :integer, default: 4

    def write(chunk)
      delete_latest_file while file_number_full?

      super chunk
    end

    private
    def file_number_full?
      Dir.glob(@path + ".*").length >= @file_number_limit
    end

    def delete_latest_file
      dir = File.dirname(@path)
      file = Dir.entries(dir).select{ |item|
        FileTest.file?(dir + "/" + item) }.map { |f|
          path = dir + "/" + f
          { path: path, time: File.mtime(path), size: File.size(path) }
        }.sort { |x, y| x[:time] <=> y[:time]}[0]

      File.delete(file[:path])
    end
  end
end
